/* Main processing code for Services.
 *
 * Blitzed Services copyright (c) 2000-2002 Blitzed Services team
 *     E-mail: services@lists.blitzed.org
 * Based on ircservices-4.4.8:
 * copyright (c) 1996-1999 Andrew Church.
 *     E-mail: <achurch@dragonfire.net>
 * copyright (c) 1999-2000 Andrew Kempe.
 *     E-mail: <theshadow@shadowfire.org>
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

#include "stdinc.h"
#include "tools.h"
#include "user.h"
#include "process.h"
#include "services.h"
#include "messages.h"
#include "mystring.h"
#include "memory.h"
#include "send.h"
#include "log.h"

/* $Id: process.c,v 1.4 2003/11/05 00:33:42 stu Exp $ */

/* Use ignore code? */
int allow_ignore = 1;
/* People to ignore (hashed by first character of nick). */
dlink_list ignore_list;

/* add_ignore: Add someone to the ignorance list for the next `delta'
 *             seconds.
 */

void 
add_ignore(const char *nick, time_t delta)
{
    struct IgnoreData *ign;
    char who[NICKMAX];
    time_t now = time(NULL);
    dlink_node *ptr;

    strscpy(who, nick, NICKMAX);
    ign = NULL;
    DLINK_FOREACH(ptr, ignore_list.head)
    {
        ign = ptr->data;
        if (stricmp(ign->who, who) == 0)
            break;
    }
    if (ign != NULL) 
    {
        if (ign->time > now)
            ign->time += delta;
        else
            ign->time = now + delta;
    } 
    else 
    {
        ign = smalloc(sizeof(*ign));
        strscpy(ign->who, who, sizeof(ign->who));
        ign->time = now + delta;
        dlinkAdd(ign, &ign->node, &ignore_list);
    }
}

/*************************************************************************/

/* get_ignore: Retrieve an ignorance record for a nick.  If the nick isn't
 *             being ignored, return NULL and flush the record from the
 *             in-core list if it exists (i.e. ignore timed out).
 */

struct IgnoreData *
get_ignore(const char *nick)
{
    struct IgnoreData *ign;
    time_t now = time(NULL);
    dlink_node *ptr;

    ign = NULL;
    DLINK_FOREACH(ptr, ignore_list.head)
    {
        ign = ptr->data;
        if (irc_stricmp(ign->who, nick) == 0)
            break;
    }
    if (ign != NULL && ign->time <= now) 
    {
        dlinkDelete(&ign->node, &ignore_list);
        free(ign);
        ign = NULL;
    }
    return ign;
}

/* split_buf:  Split a buffer into arguments and store the arguments in an
 *             argument vector pointed to by argv (which will be malloc'd
 *             as necessary); return the argument count.  If colon_special
 *             is non-zero, then treat a parameter with a leading ':' as
 *             the last parameter of the line, per the IRC RFC.  Destroys
 *             the buffer by side effect.
 */

int 
split_buf(char *buf, char ***argv, int colon_special)
{
	size_t argvsize;
	unsigned int argc;
	char *s;

	argvsize = 8;

	*argv = smalloc(sizeof(char *) * argvsize);
	argc = 0;

	while (*buf) 
    {
		if (argc == argvsize) 
        {
			argvsize += 8;
			*argv = srealloc(*argv, sizeof(char *) * argvsize);
		}
		
		if (*buf == ':') 
        {
			(*argv)[argc++] = buf+1;
			buf = "";
		} 
        else 
        {
			s = strpbrk(buf, " ");
			if (s) 
            {
				*s++ = 0;
				while (isspace(*s))
					s++;
			} 
            else 
				s = buf + strlen(buf);

			(*argv)[argc++] = buf;
			buf = s;
		}
	}
	return(argc);
}

/* process:  Main processing routine.  Takes the string in inbuf (global
 *           variable) and does something appropriate with it. */

void 
process()
{
	char buf[512];		/* Longest legal IRC command line */
	char source[64];
	char cmd[64];
	int ac;			/* Parameters for the command */
	char *s, *bufptr;
	char **av;
	struct Message *m;


	/* If debugging, slog the buffer. */
	if (debug)
		slog("debug: Received: %s", inbuf);

	/*
	 * First make a copy of the buffer so we have the original in case we
	 * crash - in that case, we want to know what we crashed on.
	 */

	/*
	 * XXX - Sometimes we seem to get a -1 byte at the start of the buffer
	 * when slog rotating.  This is a hack to get around it until I get to
	 * the bottom of it.
	 */
	bufptr = inbuf;
	if (inbuf[0] == (char)-1) 
    {
		snoop(s_OperServ, "[NET] Got that weird read of -1 from network (%s)",
                inbuf);
		slog("Got that weird read of -1 from network (%s)", inbuf);
		bufptr++;
	}

	strscpy(buf, bufptr, sizeof(buf));

	/* Split the buffer into pieces. */
	if (*buf == ':') 
    {
		s = strpbrk(buf, " ");

		if (!s)
			return;

		*s = 0;

		while (isspace(*++s))
			;

		strscpy(source, buf+1, sizeof(source));
		memmove(buf, s, strlen(s)+1);
	} 
    else 
		*source = 0;

	if (!*buf)
		return;

	s = strpbrk(buf, " ");

	if (s) 
    {
		*s = 0;
		while (isspace(*++s))
            ;
	} 
    else 
		s = buf + strlen(buf);
    
	strscpy(cmd, buf, sizeof(cmd));
	ac = split_buf(s, &av, 1);

	/* Do something with the message. */
	m = find_message(cmd);

	if (m) 
    {
		if (m->func)
			m->func(source, ac, av);
	} 
    else 
    {
		snoop(s_OperServ, "[NET] Unknown message from server (%s)", bufptr);
		slog("unknown message from server (%s)", bufptr);
	}

	/* Free argument list we created. */
	free(av);
}
